package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TAccessStoreLog;
import com.haomostudio.so.service.TAccessStoreLogService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TAccessStoreLog")
@Controller
public class TAccessStoreLogController {

    @Autowired
    TAccessStoreLogService tAccessStoreLogService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TAccessStoreLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_access_store_logs/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTAccessStoreLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "relationDate", required = false) String relationDate,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "storeName", required = false) String storeName,
            @RequestParam(value = "storeId", required = false) Integer storeId,
            @RequestParam(value = "accessTime", required = false) String accessTime,
            @RequestParam(value = "accessUserId", required = false) Integer accessUserId,
            @RequestParam(value = "channel", required = false) String channel
    ) {
        TAccessStoreLog item = new TAccessStoreLog();
        String id = Tools.getUUID();
        Integer integer = Integer.valueOf(id);
        item.setId(Integer.valueOf(id));
        if( relationDate != null ){
            item.setRelationDate(relationDate);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( storeName != null ){
            item.setStoreName(storeName);
        }
        if( storeId != null ){
            item.setStoreId(storeId);
        }
        if( accessTime != null ){
            item.setAccessTime(Tools.convertStringToDate(accessTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( accessUserId != null ){
            item.setAccessUserId(accessUserId);
        }
        if( channel != null ){
            item.setChannel(channel);
        }
        tAccessStoreLogService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tAccessStoreLogService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TAccessStoreLog'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_access_store_logs/{t_access_store_log_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTAccessStoreLog(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_access_store_log_id") Integer id
    ) {
        TAccessStoreLog item = tAccessStoreLogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tAccessStoreLogService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TAccessStoreLog'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_access_store_logs/{t_access_store_log_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTAccessStoreLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_access_store_log_id") Integer id,
            @RequestParam(value = "relationDate", required = false) String relationDate,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "storeName", required = false) String storeName,
            @RequestParam(value = "storeId", required = false) Integer storeId,
            @RequestParam(value = "accessTime", required = false) String accessTime,
            @RequestParam(value = "accessUserId", required = false) Integer accessUserId,
            @RequestParam(value = "channel", required = false) String channel
    ) {
        TAccessStoreLog item = tAccessStoreLogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( relationDate != null ){
            item.setRelationDate(relationDate);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( storeName != null ){
            item.setStoreName(storeName);
        }
        if( storeId != null ){
            item.setStoreId(storeId);
        }
        if( accessTime != null ){
            item.setAccessTime(Tools.convertStringToDate(accessTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( accessUserId != null ){
            item.setAccessUserId(accessUserId);
        }
        if( channel != null ){
            item.setChannel(channel);
        }
        tAccessStoreLogService.update(item);

        return tAccessStoreLogService.get(id);
    }

    @ApiOperation(value = "查询'TAccessStoreLog'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_access_store_logs/{t_access_store_log_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTAccessStoreLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_access_store_log_id") Integer id,
            @RequestParam(value = "relationDate", required = false) String relationDate,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "storeName", required = false) String storeName,
            @RequestParam(value = "storeId", required = false) Integer storeId,
            @RequestParam(value = "accessTime", required = false) String accessTime,
            @RequestParam(value = "accessUserId", required = false) Integer accessUserId,
            @RequestParam(value = "channel", required = false) String channel
    ) {
        if(request.getMethod().equals("GET")){
            TAccessStoreLog item = tAccessStoreLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TAccessStoreLog item = tAccessStoreLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( relationDate != null ){
                item.setRelationDate(relationDate);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( storeName != null ){
                item.setStoreName(storeName);
            }
            if( storeId != null ){
                item.setStoreId(storeId);
            }
            if( accessTime != null ){
                item.setAccessTime(Tools.convertStringToDate(accessTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( accessUserId != null ){
                item.setAccessUserId(accessUserId);
            }
            if( channel != null ){
                item.setChannel(channel);
            }
            tAccessStoreLogService.update(item);

            return tAccessStoreLogService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TAccessStoreLog item = tAccessStoreLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tAccessStoreLogService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TAccessStoreLog'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_access_store_logs",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTAccessStoreLogs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tAccessStoreLogService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tAccessStoreLogService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_access_store_logs/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTAccessStoreLogsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TAccessStoreLog> list = tAccessStoreLogService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TAccessStoreLog.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TAccessStoreLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_access_store_logs/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTAccessStoreLogs(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TAccessStoreLog> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TAccessStoreLog item = new TAccessStoreLog();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("relationDate")   ){
            item.setRelationDate(jsonObject.get("relationDate").toString());
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("storeName")   ){
            item.setStoreName(jsonObject.get("storeName").toString());
        }
        if(jsonObject.containsKey("storeId")   ){
            item.setStoreId(Integer.valueOf(jsonObject.get("storeId").toString()));
        }
        if(jsonObject.containsKey("accessTime")   ){
            item.setAccessTime(Tools.convertStringToDate(jsonObject.get("accessTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("accessUserId")   ){
            item.setAccessUserId(Integer.valueOf(jsonObject.get("accessUserId").toString()));
        }
        if(jsonObject.containsKey("channel")   ){
            item.setChannel(jsonObject.get("channel").toString());
        }
             tAccessStoreLogService.create(item);
             lists.add(tAccessStoreLogService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TAccessStoreLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_access_store_logs/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTAccessStoreLog(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TAccessStoreLog item = tAccessStoreLogService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tAccessStoreLogService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
