package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TAdminUser;
import com.haomostudio.so.service.TAdminUserService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TAdminUser")
@Controller
public class TAdminUserController {

    @Autowired
    TAdminUserService tAdminUserService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TAdminUser'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_admin_users/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTAdminUser(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "salt", required = false) String salt,
            @RequestParam(value = "number", required = false) String number,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "isSuper", required = false) Integer isSuper,
            @RequestParam(value = "headImgUrl", required = false) String headImgUrl,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "contactTel", required = false) String contactTel,
            @RequestParam(value = "email", required = false) String email,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "status", required = false) Integer status
    ) {
        TAdminUser item = new TAdminUser();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( username != null ){
            item.setUsername(username);
        }
        if( password != null ){
            item.setPassword(password);
        }
        if( salt != null ){
            item.setSalt(salt);
        }
        if( number != null ){
            item.setNumber(number);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( isSuper != null ){
            item.setIsSuper(isSuper);
        }
        if( headImgUrl != null ){
            item.setHeadImgUrl(headImgUrl);
        }
        if( name != null ){
            item.setName(name);
        }
        if( contactTel != null ){
            item.setContactTel(contactTel);
        }
        if( email != null ){
            item.setEmail(email);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( status != null ){
            item.setStatus(status);
        }
        tAdminUserService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tAdminUserService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TAdminUser'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_admin_users/{t_admin_user_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTAdminUser(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_admin_user_id") Integer id
    ) {
        TAdminUser item = tAdminUserService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tAdminUserService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TAdminUser'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_admin_users/{t_admin_user_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTAdminUser(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_admin_user_id") Integer id,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "salt", required = false) String salt,
            @RequestParam(value = "number", required = false) String number,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "isSuper", required = false) Integer isSuper,
            @RequestParam(value = "headImgUrl", required = false) String headImgUrl,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "contactTel", required = false) String contactTel,
            @RequestParam(value = "email", required = false) String email,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "status", required = false) Integer status
    ) {
        TAdminUser item = tAdminUserService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( username != null ){
            item.setUsername(username);
        }
        if( password != null ){
            item.setPassword(password);
        }
        if( salt != null ){
            item.setSalt(salt);
        }
        if( number != null ){
            item.setNumber(number);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( isSuper != null ){
            item.setIsSuper(isSuper);
        }
        if( headImgUrl != null ){
            item.setHeadImgUrl(headImgUrl);
        }
        if( name != null ){
            item.setName(name);
        }
        if( contactTel != null ){
            item.setContactTel(contactTel);
        }
        if( email != null ){
            item.setEmail(email);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( status != null ){
            item.setStatus(status);
        }
        tAdminUserService.update(item);

        return tAdminUserService.get(id);
    }

    @ApiOperation(value = "查询'TAdminUser'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_admin_users/{t_admin_user_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTAdminUser(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_admin_user_id") Integer id,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "salt", required = false) String salt,
            @RequestParam(value = "number", required = false) String number,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "isSuper", required = false) Integer isSuper,
            @RequestParam(value = "headImgUrl", required = false) String headImgUrl,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "contactTel", required = false) String contactTel,
            @RequestParam(value = "email", required = false) String email,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "status", required = false) Integer status
    ) {
        if(request.getMethod().equals("GET")){
            TAdminUser item = tAdminUserService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TAdminUser item = tAdminUserService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( username != null ){
                item.setUsername(username);
            }
            if( password != null ){
                item.setPassword(password);
            }
            if( salt != null ){
                item.setSalt(salt);
            }
            if( number != null ){
                item.setNumber(number);
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            if( updated != null ){
                item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
            }
            if( isSuper != null ){
                item.setIsSuper(isSuper);
            }
            if( headImgUrl != null ){
                item.setHeadImgUrl(headImgUrl);
            }
            if( name != null ){
                item.setName(name);
            }
            if( contactTel != null ){
                item.setContactTel(contactTel);
            }
            if( email != null ){
                item.setEmail(email);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( status != null ){
                item.setStatus(status);
            }
            tAdminUserService.update(item);

            return tAdminUserService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TAdminUser item = tAdminUserService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tAdminUserService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TAdminUser'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_admin_users",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTAdminUsers(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tAdminUserService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tAdminUserService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_admin_users/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTAdminUsersId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TAdminUser> list = tAdminUserService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TAdminUser.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TAdminUser'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_admin_users/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTAdminUsers(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TAdminUser> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TAdminUser item = new TAdminUser();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("username")   ){
            item.setUsername(jsonObject.get("username").toString());
        }
        if(jsonObject.containsKey("password")   ){
            item.setPassword(jsonObject.get("password").toString());
        }
        if(jsonObject.containsKey("salt")   ){
            item.setSalt(jsonObject.get("salt").toString());
        }
        if(jsonObject.containsKey("number")   ){
            item.setNumber(jsonObject.get("number").toString());
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("updated")   ){
            item.setUpdated(Tools.convertStringToDate(jsonObject.get("updated").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("isSuper")   ){
            item.setIsSuper(Integer.valueOf(jsonObject.get("isSuper").toString()));
        }
        if(jsonObject.containsKey("headImgUrl")   ){
            item.setHeadImgUrl(jsonObject.get("headImgUrl").toString());
        }
        if(jsonObject.containsKey("name")   ){
            item.setName(jsonObject.get("name").toString());
        }
        if(jsonObject.containsKey("contactTel")   ){
            item.setContactTel(jsonObject.get("contactTel").toString());
        }
        if(jsonObject.containsKey("email")   ){
            item.setEmail(jsonObject.get("email").toString());
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("status")   ){
            item.setStatus(Integer.valueOf(jsonObject.get("status").toString()));
        }
             tAdminUserService.create(item);
             lists.add(tAdminUserService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TAdminUser'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_admin_users/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTAdminUser(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TAdminUser item = tAdminUserService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tAdminUserService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
