package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TCreditRule;
import com.haomostudio.so.service.TCreditRuleService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TCreditRule")
@Controller
public class TCreditRuleController {

    @Autowired
    TCreditRuleService tCreditRuleService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TCreditRule'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_credit_rules/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTCreditRule(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "remark", required = false) String remark,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        TCreditRule item = new TCreditRule();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( name != null ){
            item.setName(name);
        }
        if( remark != null ){
            item.setRemark(remark);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        tCreditRuleService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tCreditRuleService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TCreditRule'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_credit_rules/{t_credit_rule_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTCreditRule(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_credit_rule_id") Integer id
    ) {
        TCreditRule item = tCreditRuleService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tCreditRuleService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TCreditRule'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_credit_rules/{t_credit_rule_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTCreditRule(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_credit_rule_id") Integer id,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "remark", required = false) String remark,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        TCreditRule item = tCreditRuleService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( name != null ){
            item.setName(name);
        }
        if( remark != null ){
            item.setRemark(remark);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        tCreditRuleService.update(item);

        return tCreditRuleService.get(id);
    }

    @ApiOperation(value = "查询'TCreditRule'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_credit_rules/{t_credit_rule_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCreditRule(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_credit_rule_id") Integer id,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "remark", required = false) String remark,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        if(request.getMethod().equals("GET")){
            TCreditRule item = tCreditRuleService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TCreditRule item = tCreditRuleService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( name != null ){
                item.setName(name);
            }
            if( remark != null ){
                item.setRemark(remark);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            if( updated != null ){
                item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
            }
            if( adminUserId != null ){
                item.setAdminUserId(adminUserId);
            }
            tCreditRuleService.update(item);

            return tCreditRuleService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TCreditRule item = tCreditRuleService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tCreditRuleService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TCreditRule'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_credit_rules",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCreditRules(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tCreditRuleService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tCreditRuleService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_credit_rules/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTCreditRulesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TCreditRule> list = tCreditRuleService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TCreditRule.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TCreditRule'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_credit_rules/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTCreditRules(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TCreditRule> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TCreditRule item = new TCreditRule();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("name")   ){
            item.setName(jsonObject.get("name").toString());
        }
        if(jsonObject.containsKey("remark")   ){
            item.setRemark(jsonObject.get("remark").toString());
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("updated")   ){
            item.setUpdated(Tools.convertStringToDate(jsonObject.get("updated").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("adminUserId")   ){
            item.setAdminUserId(Integer.valueOf(jsonObject.get("adminUserId").toString()));
        }
             tCreditRuleService.create(item);
             lists.add(tCreditRuleService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TCreditRule'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_credit_rules/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTCreditRule(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TCreditRule item = tCreditRuleService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tCreditRuleService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
