package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TFixAssertOperation;
import com.haomostudio.so.service.TFixAssertOperationService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TFixAssertOperation")
@Controller
public class TFixAssertOperationController {

    @Autowired
    TFixAssertOperationService tFixAssertOperationService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TFixAssertOperation'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_fix_assert_operations/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTFixAssertOperation(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "assertId", required = false) Integer assertId,
            @RequestParam(value = "companyId", required = false) Integer companyId,
            @RequestParam(value = "operationDate", required = false) String operationDate,
            @RequestParam(value = "operationType", required = false) Integer operationType,
            @RequestParam(value = "operationNum", required = false) Integer operationNum
    ) {
        TFixAssertOperation item = new TFixAssertOperation();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( assertId != null ){
            item.setAssertId(assertId);
        }
        if( companyId != null ){
            item.setCompanyId(companyId);
        }
        if( operationDate != null ){
            item.setOperationDate(Tools.convertStringToDate(operationDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( operationType != null ){
            item.setOperationType(operationType);
        }
        if( operationNum != null ){
            item.setOperationNum(operationNum);
        }
        tFixAssertOperationService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tFixAssertOperationService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TFixAssertOperation'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_fix_assert_operations/{t_fix_assert_operation_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTFixAssertOperation(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_fix_assert_operation_id") Integer id
    ) {
        TFixAssertOperation item = tFixAssertOperationService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tFixAssertOperationService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TFixAssertOperation'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_fix_assert_operations/{t_fix_assert_operation_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTFixAssertOperation(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_fix_assert_operation_id") Integer id,
            @RequestParam(value = "assertId", required = false) Integer assertId,
            @RequestParam(value = "companyId", required = false) Integer companyId,
            @RequestParam(value = "operationDate", required = false) String operationDate,
            @RequestParam(value = "operationType", required = false) Integer operationType,
            @RequestParam(value = "operationNum", required = false) Integer operationNum
    ) {
        TFixAssertOperation item = tFixAssertOperationService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( assertId != null ){
            item.setAssertId(assertId);
        }
        if( companyId != null ){
            item.setCompanyId(companyId);
        }
        if( operationDate != null ){
            item.setOperationDate(Tools.convertStringToDate(operationDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( operationType != null ){
            item.setOperationType(operationType);
        }
        if( operationNum != null ){
            item.setOperationNum(operationNum);
        }
        tFixAssertOperationService.update(item);

        return tFixAssertOperationService.get(id);
    }

    @ApiOperation(value = "查询'TFixAssertOperation'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_fix_assert_operations/{t_fix_assert_operation_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTFixAssertOperation(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_fix_assert_operation_id") Integer id,
            @RequestParam(value = "assertId", required = false) Integer assertId,
            @RequestParam(value = "companyId", required = false) Integer companyId,
            @RequestParam(value = "operationDate", required = false) String operationDate,
            @RequestParam(value = "operationType", required = false) Integer operationType,
            @RequestParam(value = "operationNum", required = false) Integer operationNum
    ) {
        if(request.getMethod().equals("GET")){
            TFixAssertOperation item = tFixAssertOperationService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TFixAssertOperation item = tFixAssertOperationService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( assertId != null ){
                item.setAssertId(assertId);
            }
            if( companyId != null ){
                item.setCompanyId(companyId);
            }
            if( operationDate != null ){
                item.setOperationDate(Tools.convertStringToDate(operationDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( operationType != null ){
                item.setOperationType(operationType);
            }
            if( operationNum != null ){
                item.setOperationNum(operationNum);
            }
            tFixAssertOperationService.update(item);

            return tFixAssertOperationService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TFixAssertOperation item = tFixAssertOperationService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tFixAssertOperationService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TFixAssertOperation'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_fix_assert_operations",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTFixAssertOperations(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tFixAssertOperationService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tFixAssertOperationService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_fix_assert_operations/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTFixAssertOperationsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TFixAssertOperation> list = tFixAssertOperationService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TFixAssertOperation.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TFixAssertOperation'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_fix_assert_operations/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTFixAssertOperations(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TFixAssertOperation> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TFixAssertOperation item = new TFixAssertOperation();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("assertId")   ){
            item.setAssertId(Integer.valueOf(jsonObject.get("assertId").toString()));
        }
        if(jsonObject.containsKey("companyId")   ){
            item.setCompanyId(Integer.valueOf(jsonObject.get("companyId").toString()));
        }
        if(jsonObject.containsKey("operationDate")   ){
            item.setOperationDate(Tools.convertStringToDate(jsonObject.get("operationDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("operationType")   ){
            item.setOperationType(Integer.valueOf(jsonObject.get("operationType").toString()));
        }
        if(jsonObject.containsKey("operationNum")   ){
            item.setOperationNum(Integer.valueOf(jsonObject.get("operationNum").toString()));
        }
             tFixAssertOperationService.create(item);
             lists.add(tFixAssertOperationService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TFixAssertOperation'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_fix_assert_operations/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTFixAssertOperation(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TFixAssertOperation item = tFixAssertOperationService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tFixAssertOperationService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
