package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TOperationLog;
import com.haomostudio.so.service.TOperationLogService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TOperationLog")
@Controller
public class TOperationLogController {

    @Autowired
    TOperationLogService tOperationLogService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TOperationLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_operation_logs/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTOperationLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "logModule", required = false) String logModule,
            @RequestParam(value = "logAction", required = false) String logAction,
            @RequestParam(value = "targetId", required = false) Integer targetId,
            @RequestParam(value = "logTime", required = false) String logTime,
            @RequestParam(value = "targetName", required = false) String targetName,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        TOperationLog item = new TOperationLog();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( logModule != null ){
            item.setLogModule(logModule);
        }
        if( logAction != null ){
            item.setLogAction(logAction);
        }
        if( targetId != null ){
            item.setTargetId(targetId);
        }
        if( logTime != null ){
            item.setLogTime(Tools.convertStringToDate(logTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( targetName != null ){
            item.setTargetName(targetName);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        tOperationLogService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tOperationLogService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TOperationLog'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_operation_logs/{t_operation_log_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTOperationLog(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_operation_log_id") Integer id
    ) {
        TOperationLog item = tOperationLogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tOperationLogService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TOperationLog'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_operation_logs/{t_operation_log_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTOperationLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_operation_log_id") Integer id,
            @RequestParam(value = "logModule", required = false) String logModule,
            @RequestParam(value = "logAction", required = false) String logAction,
            @RequestParam(value = "targetId", required = false) Integer targetId,
            @RequestParam(value = "logTime", required = false) String logTime,
            @RequestParam(value = "targetName", required = false) String targetName,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        TOperationLog item = tOperationLogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( logModule != null ){
            item.setLogModule(logModule);
        }
        if( logAction != null ){
            item.setLogAction(logAction);
        }
        if( targetId != null ){
            item.setTargetId(targetId);
        }
        if( logTime != null ){
            item.setLogTime(Tools.convertStringToDate(logTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( targetName != null ){
            item.setTargetName(targetName);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        tOperationLogService.update(item);

        return tOperationLogService.get(id);
    }

    @ApiOperation(value = "查询'TOperationLog'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_operation_logs/{t_operation_log_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTOperationLog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_operation_log_id") Integer id,
            @RequestParam(value = "logModule", required = false) String logModule,
            @RequestParam(value = "logAction", required = false) String logAction,
            @RequestParam(value = "targetId", required = false) Integer targetId,
            @RequestParam(value = "logTime", required = false) String logTime,
            @RequestParam(value = "targetName", required = false) String targetName,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId
    ) {
        if(request.getMethod().equals("GET")){
            TOperationLog item = tOperationLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TOperationLog item = tOperationLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( logModule != null ){
                item.setLogModule(logModule);
            }
            if( logAction != null ){
                item.setLogAction(logAction);
            }
            if( targetId != null ){
                item.setTargetId(targetId);
            }
            if( logTime != null ){
                item.setLogTime(Tools.convertStringToDate(logTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( targetName != null ){
                item.setTargetName(targetName);
            }
            if( adminUserId != null ){
                item.setAdminUserId(adminUserId);
            }
            tOperationLogService.update(item);

            return tOperationLogService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TOperationLog item = tOperationLogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tOperationLogService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TOperationLog'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_operation_logs",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTOperationLogs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tOperationLogService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tOperationLogService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_operation_logs/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTOperationLogsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TOperationLog> list = tOperationLogService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TOperationLog.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TOperationLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_operation_logs/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTOperationLogs(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TOperationLog> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TOperationLog item = new TOperationLog();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("logModule")   ){
            item.setLogModule(jsonObject.get("logModule").toString());
        }
        if(jsonObject.containsKey("logAction")   ){
            item.setLogAction(jsonObject.get("logAction").toString());
        }
        if(jsonObject.containsKey("targetId")   ){
            item.setTargetId(Integer.valueOf(jsonObject.get("targetId").toString()));
        }
        if(jsonObject.containsKey("logTime")   ){
            item.setLogTime(Tools.convertStringToDate(jsonObject.get("logTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("targetName")   ){
            item.setTargetName(jsonObject.get("targetName").toString());
        }
        if(jsonObject.containsKey("adminUserId")   ){
            item.setAdminUserId(Integer.valueOf(jsonObject.get("adminUserId").toString()));
        }
             tOperationLogService.create(item);
             lists.add(tOperationLogService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TOperationLog'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_operation_logs/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTOperationLog(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TOperationLog item = tOperationLogService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tOperationLogService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
