package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformActivityStatistics;
import com.haomostudio.so.service.TPlatformActivityStatisticsService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformActivityStatistics")
@Controller
public class TPlatformActivityStatisticsController {

    @Autowired
    TPlatformActivityStatisticsService tPlatformActivityStatisticsService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformActivityStatistics'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_statisticses/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformActivityStatistics(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "date", required = false) String date,
            @RequestParam(value = "lookPlayer", required = false) Integer lookPlayer,
            @RequestParam(value = "lotteryCount", required = false) Integer lotteryCount,
            @RequestParam(value = "sharePlayer", required = false) Integer sharePlayer,
            @RequestParam(value = "shareCount", required = false) Integer shareCount,
            @RequestParam(value = "platformActivityId", required = false) Integer platformActivityId
    ) {
        TPlatformActivityStatistics item = new TPlatformActivityStatistics();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( date != null ){
            item.setDate(Tools.convertStringToDate(date, "yyyy-MM-dd"));
        }
        if( lookPlayer != null ){
            item.setLookPlayer(lookPlayer);
        }
        if( lotteryCount != null ){
            item.setLotteryCount(lotteryCount);
        }
        if( sharePlayer != null ){
            item.setSharePlayer(sharePlayer);
        }
        if( shareCount != null ){
            item.setShareCount(shareCount);
        }
        if( platformActivityId != null ){
            item.setPlatformActivityId(platformActivityId);
        }
        tPlatformActivityStatisticsService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformActivityStatisticsService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformActivityStatistics'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_activity_statisticses/{t_platform_activity_statistics_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformActivityStatistics(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_statistics_id") Integer id
    ) {
        TPlatformActivityStatistics item = tPlatformActivityStatisticsService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformActivityStatisticsService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformActivityStatistics'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_statisticses/{t_platform_activity_statistics_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformActivityStatistics(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_statistics_id") Integer id,
            @RequestParam(value = "date", required = false) String date,
            @RequestParam(value = "lookPlayer", required = false) Integer lookPlayer,
            @RequestParam(value = "lotteryCount", required = false) Integer lotteryCount,
            @RequestParam(value = "sharePlayer", required = false) Integer sharePlayer,
            @RequestParam(value = "shareCount", required = false) Integer shareCount,
            @RequestParam(value = "platformActivityId", required = false) Integer platformActivityId
    ) {
        TPlatformActivityStatistics item = tPlatformActivityStatisticsService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( date != null ){
            item.setDate(Tools.convertStringToDate(date, "yyyy-MM-dd"));
        }
        if( lookPlayer != null ){
            item.setLookPlayer(lookPlayer);
        }
        if( lotteryCount != null ){
            item.setLotteryCount(lotteryCount);
        }
        if( sharePlayer != null ){
            item.setSharePlayer(sharePlayer);
        }
        if( shareCount != null ){
            item.setShareCount(shareCount);
        }
        if( platformActivityId != null ){
            item.setPlatformActivityId(platformActivityId);
        }
        tPlatformActivityStatisticsService.update(item);

        return tPlatformActivityStatisticsService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformActivityStatistics'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_activity_statisticses/{t_platform_activity_statistics_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityStatistics(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_activity_statistics_id") Integer id,
            @RequestParam(value = "date", required = false) String date,
            @RequestParam(value = "lookPlayer", required = false) Integer lookPlayer,
            @RequestParam(value = "lotteryCount", required = false) Integer lotteryCount,
            @RequestParam(value = "sharePlayer", required = false) Integer sharePlayer,
            @RequestParam(value = "shareCount", required = false) Integer shareCount,
            @RequestParam(value = "platformActivityId", required = false) Integer platformActivityId
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformActivityStatistics item = tPlatformActivityStatisticsService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformActivityStatistics item = tPlatformActivityStatisticsService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( date != null ){
                item.setDate(Tools.convertStringToDate(date, "yyyy-MM-dd"));
            }
            if( lookPlayer != null ){
                item.setLookPlayer(lookPlayer);
            }
            if( lotteryCount != null ){
                item.setLotteryCount(lotteryCount);
            }
            if( sharePlayer != null ){
                item.setSharePlayer(sharePlayer);
            }
            if( shareCount != null ){
                item.setShareCount(shareCount);
            }
            if( platformActivityId != null ){
                item.setPlatformActivityId(platformActivityId);
            }
            tPlatformActivityStatisticsService.update(item);

            return tPlatformActivityStatisticsService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformActivityStatistics item = tPlatformActivityStatisticsService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformActivityStatisticsService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformActivityStatistics'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_activity_statisticses",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityStatisticses(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformActivityStatisticsService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformActivityStatisticsService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_activity_statisticses/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformActivityStatisticsesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformActivityStatistics> list = tPlatformActivityStatisticsService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformActivityStatistics.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformActivityStatistics'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_activity_statisticses/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformActivityStatisticses(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformActivityStatistics> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformActivityStatistics item = new TPlatformActivityStatistics();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("date")   ){
            item.setDate(Tools.convertStringToDate(jsonObject.get("date").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("lookPlayer")   ){
            item.setLookPlayer(Integer.valueOf(jsonObject.get("lookPlayer").toString()));
        }
        if(jsonObject.containsKey("lotteryCount")   ){
            item.setLotteryCount(Integer.valueOf(jsonObject.get("lotteryCount").toString()));
        }
        if(jsonObject.containsKey("sharePlayer")   ){
            item.setSharePlayer(Integer.valueOf(jsonObject.get("sharePlayer").toString()));
        }
        if(jsonObject.containsKey("shareCount")   ){
            item.setShareCount(Integer.valueOf(jsonObject.get("shareCount").toString()));
        }
        if(jsonObject.containsKey("platformActivityId")   ){
            item.setPlatformActivityId(Integer.valueOf(jsonObject.get("platformActivityId").toString()));
        }
             tPlatformActivityStatisticsService.create(item);
             lists.add(tPlatformActivityStatisticsService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformActivityStatistics'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_activity_statisticses/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformActivityStatistics(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformActivityStatistics item = tPlatformActivityStatisticsService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformActivityStatisticsService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
