package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformField;
import com.haomostudio.so.service.TPlatformFieldService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformField")
@Controller
public class TPlatformFieldController {

    @Autowired
    TPlatformFieldService tPlatformFieldService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformField'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_fields/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformField(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "fieldId", required = false) Integer fieldId,
            @RequestParam(value = "fieldType", required = false) Integer fieldType,
            @RequestParam(value = "storeId", required = false) Integer storeId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "spaceOpenStatus", required = false) Integer spaceOpenStatus,
            @RequestParam(value = "platformIsOpen", required = false) Integer platformIsOpen,
            @RequestParam(value = "bookNum", required = false) Integer bookNum,
            @RequestParam(value = "setOpenTime", required = false) String setOpenTime,
            @RequestParam(value = "isMemberDiscount", required = false) Integer isMemberDiscount,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "deleteTime", required = false) String deleteTime
    ) {
        TPlatformField item = new TPlatformField();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( fieldId != null ){
            item.setFieldId(fieldId);
        }
        if( fieldType != null ){
            item.setFieldType(fieldType);
        }
        if( storeId != null ){
            item.setStoreId(storeId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( spaceOpenStatus != null ){
            item.setSpaceOpenStatus(spaceOpenStatus);
        }
        if( platformIsOpen != null ){
            item.setPlatformIsOpen(platformIsOpen);
        }
        if( bookNum != null ){
            item.setBookNum(bookNum);
        }
        if( setOpenTime != null ){
            item.setSetOpenTime(Tools.convertStringToDate(setOpenTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( isMemberDiscount != null ){
            item.setIsMemberDiscount(isMemberDiscount);
        }
        if( isDelete != null ){
            item.setIsDelete(isDelete);
        }
        if( deleteTime != null ){
            item.setDeleteTime(Tools.convertStringToDate(deleteTime, "yyyy-MM-dd HH:mm:ss"));
        }
        tPlatformFieldService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformFieldService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformField'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_fields/{t_platform_field_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformField(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_field_id") Integer id
    ) {
        TPlatformField item = tPlatformFieldService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformFieldService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformField'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_fields/{t_platform_field_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformField(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_field_id") Integer id,
            @RequestParam(value = "fieldId", required = false) Integer fieldId,
            @RequestParam(value = "fieldType", required = false) Integer fieldType,
            @RequestParam(value = "storeId", required = false) Integer storeId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "spaceOpenStatus", required = false) Integer spaceOpenStatus,
            @RequestParam(value = "platformIsOpen", required = false) Integer platformIsOpen,
            @RequestParam(value = "bookNum", required = false) Integer bookNum,
            @RequestParam(value = "setOpenTime", required = false) String setOpenTime,
            @RequestParam(value = "isMemberDiscount", required = false) Integer isMemberDiscount,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "deleteTime", required = false) String deleteTime
    ) {
        TPlatformField item = tPlatformFieldService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( fieldId != null ){
            item.setFieldId(fieldId);
        }
        if( fieldType != null ){
            item.setFieldType(fieldType);
        }
        if( storeId != null ){
            item.setStoreId(storeId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( spaceOpenStatus != null ){
            item.setSpaceOpenStatus(spaceOpenStatus);
        }
        if( platformIsOpen != null ){
            item.setPlatformIsOpen(platformIsOpen);
        }
        if( bookNum != null ){
            item.setBookNum(bookNum);
        }
        if( setOpenTime != null ){
            item.setSetOpenTime(Tools.convertStringToDate(setOpenTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( isMemberDiscount != null ){
            item.setIsMemberDiscount(isMemberDiscount);
        }
        if( isDelete != null ){
            item.setIsDelete(isDelete);
        }
        if( deleteTime != null ){
            item.setDeleteTime(Tools.convertStringToDate(deleteTime, "yyyy-MM-dd HH:mm:ss"));
        }
        tPlatformFieldService.update(item);

        return tPlatformFieldService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformField'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_fields/{t_platform_field_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformField(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_field_id") Integer id,
            @RequestParam(value = "fieldId", required = false) Integer fieldId,
            @RequestParam(value = "fieldType", required = false) Integer fieldType,
            @RequestParam(value = "storeId", required = false) Integer storeId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "spaceOpenStatus", required = false) Integer spaceOpenStatus,
            @RequestParam(value = "platformIsOpen", required = false) Integer platformIsOpen,
            @RequestParam(value = "bookNum", required = false) Integer bookNum,
            @RequestParam(value = "setOpenTime", required = false) String setOpenTime,
            @RequestParam(value = "isMemberDiscount", required = false) Integer isMemberDiscount,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "deleteTime", required = false) String deleteTime
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformField item = tPlatformFieldService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformField item = tPlatformFieldService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( fieldId != null ){
                item.setFieldId(fieldId);
            }
            if( fieldType != null ){
                item.setFieldType(fieldType);
            }
            if( storeId != null ){
                item.setStoreId(storeId);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( spaceOpenStatus != null ){
                item.setSpaceOpenStatus(spaceOpenStatus);
            }
            if( platformIsOpen != null ){
                item.setPlatformIsOpen(platformIsOpen);
            }
            if( bookNum != null ){
                item.setBookNum(bookNum);
            }
            if( setOpenTime != null ){
                item.setSetOpenTime(Tools.convertStringToDate(setOpenTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( isMemberDiscount != null ){
                item.setIsMemberDiscount(isMemberDiscount);
            }
            if( isDelete != null ){
                item.setIsDelete(isDelete);
            }
            if( deleteTime != null ){
                item.setDeleteTime(Tools.convertStringToDate(deleteTime, "yyyy-MM-dd HH:mm:ss"));
            }
            tPlatformFieldService.update(item);

            return tPlatformFieldService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformField item = tPlatformFieldService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformFieldService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformField'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_fields",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformFields(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformFieldService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformFieldService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_fields/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformFieldsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformField> list = tPlatformFieldService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformField.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformField'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_fields/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformFields(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformField> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformField item = new TPlatformField();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("fieldId")   ){
            item.setFieldId(Integer.valueOf(jsonObject.get("fieldId").toString()));
        }
        if(jsonObject.containsKey("fieldType")   ){
            item.setFieldType(Integer.valueOf(jsonObject.get("fieldType").toString()));
        }
        if(jsonObject.containsKey("storeId")   ){
            item.setStoreId(Integer.valueOf(jsonObject.get("storeId").toString()));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("spaceOpenStatus")   ){
            item.setSpaceOpenStatus(Integer.valueOf(jsonObject.get("spaceOpenStatus").toString()));
        }
        if(jsonObject.containsKey("platformIsOpen")   ){
            item.setPlatformIsOpen(Integer.valueOf(jsonObject.get("platformIsOpen").toString()));
        }
        if(jsonObject.containsKey("bookNum")   ){
            item.setBookNum(Integer.valueOf(jsonObject.get("bookNum").toString()));
        }
        if(jsonObject.containsKey("setOpenTime")   ){
            item.setSetOpenTime(Tools.convertStringToDate(jsonObject.get("setOpenTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("isMemberDiscount")   ){
            item.setIsMemberDiscount(Integer.valueOf(jsonObject.get("isMemberDiscount").toString()));
        }
        if(jsonObject.containsKey("isDelete")   ){
            item.setIsDelete(Integer.valueOf(jsonObject.get("isDelete").toString()));
        }
        if(jsonObject.containsKey("deleteTime")   ){
            item.setDeleteTime(Tools.convertStringToDate(jsonObject.get("deleteTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
             tPlatformFieldService.create(item);
             lists.add(tPlatformFieldService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformField'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_fields/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformField(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformField item = tPlatformFieldService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformFieldService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
