package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformInvite;
import com.haomostudio.so.service.TPlatformInviteService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformInvite")
@Controller
public class TPlatformInviteController {

    @Autowired
    TPlatformInviteService tPlatformInviteService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformInvite'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_invites/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformInvite(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "platformOrderId", required = false) Integer platformOrderId,
            @RequestParam(value = "fieldType", required = false) Integer fieldType,
            @RequestParam(value = "wxMpCustomerId", required = false) Integer wxMpCustomerId,
            @RequestParam(value = "wxappCustomerId", required = false) Integer wxappCustomerId,
            @RequestParam(value = "isJoined", required = false) Integer isJoined,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "reminderSendStatus", required = false) Integer reminderSendStatus
    ) {
        TPlatformInvite item = new TPlatformInvite();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( platformOrderId != null ){
            item.setPlatformOrderId(platformOrderId);
        }
        if( fieldType != null ){
            item.setFieldType(fieldType);
        }
        if( wxMpCustomerId != null ){
            item.setWxMpCustomerId(wxMpCustomerId);
        }
        if( wxappCustomerId != null ){
            item.setWxappCustomerId(wxappCustomerId);
        }
        if( isJoined != null ){
            item.setIsJoined(isJoined);
        }
        if( createDate != null ){
            item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( reminderSendStatus != null ){
            item.setReminderSendStatus(reminderSendStatus);
        }
        tPlatformInviteService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformInviteService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformInvite'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_invites/{t_platform_invite_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformInvite(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_invite_id") Integer id
    ) {
        TPlatformInvite item = tPlatformInviteService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformInviteService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformInvite'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_invites/{t_platform_invite_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformInvite(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_invite_id") Integer id,
            @RequestParam(value = "platformOrderId", required = false) Integer platformOrderId,
            @RequestParam(value = "fieldType", required = false) Integer fieldType,
            @RequestParam(value = "wxMpCustomerId", required = false) Integer wxMpCustomerId,
            @RequestParam(value = "wxappCustomerId", required = false) Integer wxappCustomerId,
            @RequestParam(value = "isJoined", required = false) Integer isJoined,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "reminderSendStatus", required = false) Integer reminderSendStatus
    ) {
        TPlatformInvite item = tPlatformInviteService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( platformOrderId != null ){
            item.setPlatformOrderId(platformOrderId);
        }
        if( fieldType != null ){
            item.setFieldType(fieldType);
        }
        if( wxMpCustomerId != null ){
            item.setWxMpCustomerId(wxMpCustomerId);
        }
        if( wxappCustomerId != null ){
            item.setWxappCustomerId(wxappCustomerId);
        }
        if( isJoined != null ){
            item.setIsJoined(isJoined);
        }
        if( createDate != null ){
            item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( reminderSendStatus != null ){
            item.setReminderSendStatus(reminderSendStatus);
        }
        tPlatformInviteService.update(item);

        return tPlatformInviteService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformInvite'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_invites/{t_platform_invite_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformInvite(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_invite_id") Integer id,
            @RequestParam(value = "platformOrderId", required = false) Integer platformOrderId,
            @RequestParam(value = "fieldType", required = false) Integer fieldType,
            @RequestParam(value = "wxMpCustomerId", required = false) Integer wxMpCustomerId,
            @RequestParam(value = "wxappCustomerId", required = false) Integer wxappCustomerId,
            @RequestParam(value = "isJoined", required = false) Integer isJoined,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "reminderSendStatus", required = false) Integer reminderSendStatus
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformInvite item = tPlatformInviteService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformInvite item = tPlatformInviteService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( platformOrderId != null ){
                item.setPlatformOrderId(platformOrderId);
            }
            if( fieldType != null ){
                item.setFieldType(fieldType);
            }
            if( wxMpCustomerId != null ){
                item.setWxMpCustomerId(wxMpCustomerId);
            }
            if( wxappCustomerId != null ){
                item.setWxappCustomerId(wxappCustomerId);
            }
            if( isJoined != null ){
                item.setIsJoined(isJoined);
            }
            if( createDate != null ){
                item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( reminderSendStatus != null ){
                item.setReminderSendStatus(reminderSendStatus);
            }
            tPlatformInviteService.update(item);

            return tPlatformInviteService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformInvite item = tPlatformInviteService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformInviteService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformInvite'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_invites",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformInvites(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformInviteService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformInviteService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_invites/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformInvitesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformInvite> list = tPlatformInviteService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformInvite.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformInvite'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_invites/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformInvites(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformInvite> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformInvite item = new TPlatformInvite();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("platformOrderId")   ){
            item.setPlatformOrderId(Integer.valueOf(jsonObject.get("platformOrderId").toString()));
        }
        if(jsonObject.containsKey("fieldType")   ){
            item.setFieldType(Integer.valueOf(jsonObject.get("fieldType").toString()));
        }
        if(jsonObject.containsKey("wxMpCustomerId")   ){
            item.setWxMpCustomerId(Integer.valueOf(jsonObject.get("wxMpCustomerId").toString()));
        }
        if(jsonObject.containsKey("wxappCustomerId")   ){
            item.setWxappCustomerId(Integer.valueOf(jsonObject.get("wxappCustomerId").toString()));
        }
        if(jsonObject.containsKey("isJoined")   ){
            item.setIsJoined(Integer.valueOf(jsonObject.get("isJoined").toString()));
        }
        if(jsonObject.containsKey("createDate")   ){
            item.setCreateDate(Tools.convertStringToDate(jsonObject.get("createDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("reminderSendStatus")   ){
            item.setReminderSendStatus(Integer.valueOf(jsonObject.get("reminderSendStatus").toString()));
        }
             tPlatformInviteService.create(item);
             lists.add(tPlatformInviteService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformInvite'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_invites/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformInvite(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformInvite item = tPlatformInviteService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformInviteService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
