package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TSpaceKeeper;
import com.haomostudio.so.service.TSpaceKeeperService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TSpaceKeeper")
@Controller
public class TSpaceKeeperController {

    @Autowired
    TSpaceKeeperService tSpaceKeeperService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TSpaceKeeper'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_space_keepers/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTSpaceKeeper(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "createTime", required = false) String createTime,
            @RequestParam(value = "modifiedTime", required = false) String modifiedTime
    ) {
        TSpaceKeeper item = new TSpaceKeeper();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( name != null ){
            item.setName(name);
        }
        if( phone != null ){
            item.setPhone(phone);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        if( statusCode != null ){
            item.setStatusCode(statusCode);
        }
        if( isDelete != null ){
            item.setIsDelete(isDelete);
        }
            item.setCreateTime(new Date());
        if( modifiedTime != null ){
            item.setModifiedTime(Tools.convertStringToDate(modifiedTime, "yyyy-MM-dd HH:mm:ss"));
        }
        tSpaceKeeperService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tSpaceKeeperService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TSpaceKeeper'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_space_keepers/{t_space_keeper_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTSpaceKeeper(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_space_keeper_id") Integer id
    ) {
        TSpaceKeeper item = tSpaceKeeperService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tSpaceKeeperService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TSpaceKeeper'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_space_keepers/{t_space_keeper_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTSpaceKeeper(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_space_keeper_id") Integer id,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "modifiedTime", required = false) String modifiedTime
    ) {
        TSpaceKeeper item = tSpaceKeeperService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( name != null ){
            item.setName(name);
        }
        if( phone != null ){
            item.setPhone(phone);
        }
        if( adminUserId != null ){
            item.setAdminUserId(adminUserId);
        }
        if( statusCode != null ){
            item.setStatusCode(statusCode);
        }
        if( isDelete != null ){
            item.setIsDelete(isDelete);
        }
        if( modifiedTime != null ){
            item.setModifiedTime(Tools.convertStringToDate(modifiedTime, "yyyy-MM-dd HH:mm:ss"));
        }
        tSpaceKeeperService.update(item);

        return tSpaceKeeperService.get(id);
    }

    @ApiOperation(value = "查询'TSpaceKeeper'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_space_keepers/{t_space_keeper_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTSpaceKeeper(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_space_keeper_id") Integer id,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "adminUserId", required = false) Integer adminUserId,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "modifiedTime", required = false) String modifiedTime
    ) {
        if(request.getMethod().equals("GET")){
            TSpaceKeeper item = tSpaceKeeperService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TSpaceKeeper item = tSpaceKeeperService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( name != null ){
                item.setName(name);
            }
            if( phone != null ){
                item.setPhone(phone);
            }
            if( adminUserId != null ){
                item.setAdminUserId(adminUserId);
            }
            if( statusCode != null ){
                item.setStatusCode(statusCode);
            }
            if( isDelete != null ){
                item.setIsDelete(isDelete);
            }
            if( modifiedTime != null ){
                item.setModifiedTime(Tools.convertStringToDate(modifiedTime, "yyyy-MM-dd HH:mm:ss"));
            }
            tSpaceKeeperService.update(item);

            return tSpaceKeeperService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TSpaceKeeper item = tSpaceKeeperService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tSpaceKeeperService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TSpaceKeeper'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_space_keepers",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTSpaceKeepers(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tSpaceKeeperService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tSpaceKeeperService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_space_keepers/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTSpaceKeepersId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TSpaceKeeper> list = tSpaceKeeperService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TSpaceKeeper.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TSpaceKeeper'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_space_keepers/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTSpaceKeepers(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TSpaceKeeper> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TSpaceKeeper item = new TSpaceKeeper();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("name")   ){
            item.setName(jsonObject.get("name").toString());
        }
        if(jsonObject.containsKey("phone")   ){
            item.setPhone(jsonObject.get("phone").toString());
        }
        if(jsonObject.containsKey("adminUserId")   ){
            item.setAdminUserId(Integer.valueOf(jsonObject.get("adminUserId").toString()));
        }
        if(jsonObject.containsKey("statusCode")   ){
            item.setStatusCode(Integer.valueOf(jsonObject.get("statusCode").toString()));
        }
        if(jsonObject.containsKey("isDelete")   ){
            item.setIsDelete(Integer.valueOf(jsonObject.get("isDelete").toString()));
        }
            item.setCreateTime(new Date());
        if(jsonObject.containsKey("modifiedTime")   ){
            item.setModifiedTime(Tools.convertStringToDate(jsonObject.get("modifiedTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
             tSpaceKeeperService.create(item);
             lists.add(tSpaceKeeperService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TSpaceKeeper'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_space_keepers/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTSpaceKeeper(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TSpaceKeeper item = tSpaceKeeperService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tSpaceKeeperService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
