package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TTabletVersion;
import com.haomostudio.so.service.TTabletVersionService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TTabletVersion")
@Controller
public class TTabletVersionController {

    @Autowired
    TTabletVersionService tTabletVersionService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TTabletVersion'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_tablet_versions/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTTabletVersion(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "versionName", required = false) String versionName,
            @RequestParam(value = "versionCode", required = false) Integer versionCode,
            @RequestParam(value = "url", required = false) String url,
            @RequestParam(value = "isForceUpdate", required = false) Integer isForceUpdate,
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "content", required = false) String content,
            @RequestParam(value = "clientType", required = false) String clientType
    ) {
        TTabletVersion item = new TTabletVersion();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( versionName != null ){
            item.setVersionName(versionName);
        }
        if( versionCode != null ){
            item.setVersionCode(versionCode);
        }
        if( url != null ){
            item.setUrl(url);
        }
        if( isForceUpdate != null ){
            item.setIsForceUpdate(isForceUpdate);
        }
        if( title != null ){
            item.setTitle(title);
        }
        if( content != null ){
            item.setContent(content);
        }
        if( clientType != null ){
            item.setClientType(clientType);
        }
        tTabletVersionService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tTabletVersionService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TTabletVersion'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_tablet_versions/{t_tablet_version_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTTabletVersion(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_tablet_version_id") Integer id
    ) {
        TTabletVersion item = tTabletVersionService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tTabletVersionService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TTabletVersion'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_tablet_versions/{t_tablet_version_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTTabletVersion(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_tablet_version_id") Integer id,
            @RequestParam(value = "versionName", required = false) String versionName,
            @RequestParam(value = "versionCode", required = false) Integer versionCode,
            @RequestParam(value = "url", required = false) String url,
            @RequestParam(value = "isForceUpdate", required = false) Integer isForceUpdate,
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "content", required = false) String content,
            @RequestParam(value = "clientType", required = false) String clientType
    ) {
        TTabletVersion item = tTabletVersionService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( versionName != null ){
            item.setVersionName(versionName);
        }
        if( versionCode != null ){
            item.setVersionCode(versionCode);
        }
        if( url != null ){
            item.setUrl(url);
        }
        if( isForceUpdate != null ){
            item.setIsForceUpdate(isForceUpdate);
        }
        if( title != null ){
            item.setTitle(title);
        }
        if( content != null ){
            item.setContent(content);
        }
        if( clientType != null ){
            item.setClientType(clientType);
        }
        tTabletVersionService.update(item);

        return tTabletVersionService.get(id);
    }

    @ApiOperation(value = "查询'TTabletVersion'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_tablet_versions/{t_tablet_version_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTTabletVersion(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_tablet_version_id") Integer id,
            @RequestParam(value = "versionName", required = false) String versionName,
            @RequestParam(value = "versionCode", required = false) Integer versionCode,
            @RequestParam(value = "url", required = false) String url,
            @RequestParam(value = "isForceUpdate", required = false) Integer isForceUpdate,
            @RequestParam(value = "title", required = false) String title,
            @RequestParam(value = "content", required = false) String content,
            @RequestParam(value = "clientType", required = false) String clientType
    ) {
        if(request.getMethod().equals("GET")){
            TTabletVersion item = tTabletVersionService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TTabletVersion item = tTabletVersionService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( versionName != null ){
                item.setVersionName(versionName);
            }
            if( versionCode != null ){
                item.setVersionCode(versionCode);
            }
            if( url != null ){
                item.setUrl(url);
            }
            if( isForceUpdate != null ){
                item.setIsForceUpdate(isForceUpdate);
            }
            if( title != null ){
                item.setTitle(title);
            }
            if( content != null ){
                item.setContent(content);
            }
            if( clientType != null ){
                item.setClientType(clientType);
            }
            tTabletVersionService.update(item);

            return tTabletVersionService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TTabletVersion item = tTabletVersionService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tTabletVersionService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TTabletVersion'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_tablet_versions",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTTabletVersions(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tTabletVersionService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tTabletVersionService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_tablet_versions/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTTabletVersionsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TTabletVersion> list = tTabletVersionService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TTabletVersion.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TTabletVersion'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_tablet_versions/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTTabletVersions(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TTabletVersion> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TTabletVersion item = new TTabletVersion();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("versionName")   ){
            item.setVersionName(jsonObject.get("versionName").toString());
        }
        if(jsonObject.containsKey("versionCode")   ){
            item.setVersionCode(Integer.valueOf(jsonObject.get("versionCode").toString()));
        }
        if(jsonObject.containsKey("url")   ){
            item.setUrl(jsonObject.get("url").toString());
        }
        if(jsonObject.containsKey("isForceUpdate")   ){
            item.setIsForceUpdate(Integer.valueOf(jsonObject.get("isForceUpdate").toString()));
        }
        if(jsonObject.containsKey("title")   ){
            item.setTitle(jsonObject.get("title").toString());
        }
        if(jsonObject.containsKey("content")   ){
            item.setContent(jsonObject.get("content").toString());
        }
        if(jsonObject.containsKey("clientType")   ){
            item.setClientType(jsonObject.get("clientType").toString());
        }
             tTabletVersionService.create(item);
             lists.add(tTabletVersionService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TTabletVersion'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_tablet_versions/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTTabletVersion(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TTabletVersion item = tTabletVersionService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tTabletVersionService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
